/**
 * Copyright 2008 Peach Jean Solutions
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.xj4.parameterized;

import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.lang.reflect.Method;
import java.util.List;
import java.util.ArrayList;

/**
 * A set of utilities for working with and manipulating parameter sets.  Intended for use internally.
 *
 * @author jbunting
 */
public class InternalParameterSetUtils extends ParameterSetUtils {

  public static ParameterSet parameterSetOf(Method method) {
    return parameterSetOf(parameterized(method), method.getDeclaringClass());
  }

  public static ParameterSet parameterSetOf(Parameterized parameterized, Class<?> targetClass) {
    String name = parameterized.value();

    return fromName(name, targetClass);
  }

  public static ParameterSet fromName(String name, Class<?> targetClass) {
    if(Parameterized.SINGLETON_NAME.equals(name)) {
      return singletonParameterSet(targetClass);
    }
    try {
      Field field = targetClass.getField(name);
      if(field == null) {
        throw new InvalidParameterSetException("Field for " + name + " does not exist.");
      }
      return fromField(field);
    } catch(InvalidParameterSetException ex) {
      throw ex;
    } catch(Exception ex) {
      throw new InvalidParameterSetException("Field " + name + " is not a valid parameter set!", ex);
    }
  }

  public static ParameterSet fromField(Field field) {
    validate(field);
    try {
      String[] labels = null;
      ParameterSet parameterSet;
      if(field.isAnnotationPresent(ParameterSet.As.class)) {
        ParameterSet.As as = field.getAnnotation(ParameterSet.As.class);
        labels = as.labels();
        if(Object[][].class.isAssignableFrom(field.getType())) {
          Object[][] values = (Object[][])field.get(null);
          parameterSet = create(values, as.labelsIncluded());
        } else if(Object[].class.isAssignableFrom(field.getType())) {
          parameterSet = create((Object[])field.get(null));
        } else {
          parameterSet = create(field.get(null));
        }
      } else if(ParameterSet.class.isAssignableFrom(field.getType())) {
        if(field.isAnnotationPresent(ParameterSet.Labels.class)) {
          labels = field.getAnnotation(ParameterSet.Labels.class).value();
        }
        parameterSet = (ParameterSet) field.get(null);
      } else {
        throw new InvalidParameterSetException("Field " + field.getName() + " is not a valid type!");
      }
      parameterSet.setName(field.getName());
      if(labels != null) {
        parameterSet.setLabels(labels);
      }
      return validate(parameterSet);
    } catch(IllegalAccessException e) {
      throw new IllegalArgumentException("Field " + field.getName() + " must be accessible.");
    }
  }

  public static void validate(Field field) {
    if(!Modifier.isStatic(field.getModifiers())) {
      throw new InvalidParameterSetException("Field " + field.getName() + " must be static!");
    }
    if(!Modifier.isFinal(field.getModifiers())) {
      throw new InvalidParameterSetException("Field " + field.getName() + " must be final!");
    }
    if(!ParameterSet.class.isAssignableFrom(field.getType()) && !field.isAnnotationPresent(ParameterSet.As.class)) {
      throw new InvalidParameterSetException("Field " + field.getName() + " must be a valid parameter set!");
    }
  }

  public static boolean isParameterSet(Field field) {
    return ParameterSet.class.isAssignableFrom(field.getType()) || field.isAnnotationPresent(ParameterSet.As.class);
  }

  public static boolean isParameterSet(String name, Class<?> targetClass) {
    try {
      return isParameterSet(targetClass.getField(name)) || Parameterized.SINGLETON_NAME.equals(name) && (singletonParameterSet(targetClass) != null);
    } catch(NoSuchFieldException e) {
      return false;
    }
  }

  public static boolean isParameterSet(Parameterized parameterized, Class<?> targetClass) {
    return isParameterSet(parameterized.value(), targetClass);
  }

  public static boolean isParameterized(Method method) {
    return method.isAnnotationPresent(Parameterized.class);
  }

  public static Parameterized parameterized(Method method) {
    return method.getAnnotation(Parameterized.class);
  }

  public static ParameterSet[] allParameterSets(Class<?> testClass) {
    List<ParameterSet> setList = new ArrayList<ParameterSet>();
    for(Field field: testClass.getFields()) {
      if(isParameterSet(field)) {
        setList.add(fromField(field));
      }
    }
    return setList.toArray(new ParameterSet[setList.size()]);
  }

  public static ParameterSet singletonParameterSet(Class<?> testClass) {
    ParameterSet[] allSets = allParameterSets(testClass);
    if(allSets.length == 1) {
      return allSets[0];
    } else {
      throw new InvalidParameterSetException("Class " + testClass.getName() + " does not have a singleton parameter set.  In order to have a singleton parameter set, one and only one parameter set must be defined.");
    }
  }

  @SuppressWarnings({"unchecked"})
  public static Object getActualParameter(Object input, Object testObject) {
    return input instanceof ParameterSource ? ((ParameterSource)input).generate(testObject) : input;
  }

  public static Class<?> getActualParameterType(Object input) {
    return input == null ? null : (input instanceof ParameterSource ? ((ParameterSource)input).getType() : input.getClass());
  }
}


