/**
 * Copyright 2008 Peach Jean Solutions
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.xj4.parameterized;

import java.lang.annotation.Documented;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;
import java.lang.annotation.ElementType;
import java.lang.annotation.Inherited;

/**
 * Interface defined by a parameter set.  The parameterized method handler uses parameter sets to invoke multiple
 * iterations of methods.
 *
 * @author Jared Bunting
 */
public interface ParameterSet {

  /**
   * Gets the name of the parameter set.
   * @return the name
   */
  String getName();

  /**
   * Gets the number of iterations present in this parameter set.
   * @return the number of iterations
   */
  int getNumIterations();

  /**
   * Gets the number of parameters provided by each iteration of the parameter set.
   * @return the size of an iteration
   */
  int getIterationSize();

  /**
   * Returns the actual parameters for the requested iteration.
   * @param iteration the iteration to return parameters for
   * @return the parameters
   */
  Object[] getIteration(int iteration);

  /**
   * Returns the label for a given iteration.  Labels are used to provide more descriptive error messages in test
   * failures.
   * @param iteration the iteration to return the label for
   * @return the label
   */
  String getIterationLabel(int iteration);

  /**
   * Validate that this parameter set is valid.  Will throw an exception if not.
   */
  void validate();

  /**
   * Defines the labels for this parameter set.  The provided string array may be shorter than the number of iterations,
   * in which case the default labeling of the parameter set will be used for unlabeled iterations.
   * @param labels the iteration labels
   */
  void setLabels(String[] labels);

  /**
   * Defines the name for this parameter set.
   * @param name the name
   */
  void setName(String name);

  /**
 * Indicates that a field represents a parameter set.  The field must be public, static, and final.  If the field is an
   * Object[][], columns represent method parameters and rows represent iterations.  If the field is an Object[], it is
   * treated as multiple iterations of a single parameter.  And, if the field is anything else, other than an
   * implementation of {@link ParameterSet}, it is treated as a single iteration of a single parameter.
   */

  @Documented
  @Retention(RetentionPolicy.RUNTIME)
  @Target(ElementType.FIELD)
  @Inherited
  public static @interface As {

    /**
     * The labels that will be used in naming the iterations of the test.  Any iterations that don't have an apporpriate
     * label will be named with their iteration index.
     * @return the labels
     */
    public abstract String[] labels() default {};

    /**
     * If this is set to true, then the engine will assume that the first column in the specified object array is the
     * labels and will not treat them as parameters.  Note that this field will be ignored if this annotation is
     * placed on a field that does not represent a two dimensional array.
     * @return true if the first column should be treated as the labels for the parameter set
     */
    public boolean labelsIncluded() default false;
  }

  /**
   * Alternative mechanism for defining labels for iterations of a parameter set.  Should be applied to a field that is
   * a parameter set.
   */
  @Documented
  @Retention(RetentionPolicy.RUNTIME)
  @Target(ElementType.FIELD)
  @Inherited
  public static @interface Labels {

    /**
     * The iteration labels for the parameter set.
     * @return the labels
     */
    public abstract String[] value() default {};
  }
}
