/**
 * Copyright 2008 Peach Jean Solutions
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.xj4.parameterized;

import org.apache.commons.lang.ArrayUtils;
import org.xj4.parameterized.set.*;

/**
 * Utility methods for creating parameter sets.
 *
 * @author jbunting
 */
public class ParameterSetUtils {

  /**
   * Creates a parameter set based on the object array.  Rows are iterations, columns are parameters.
   * @param values the parameter set values
   * @return the created parameter set
   * @see org.xj4.parameterized.set.MultiValueParameterSet
   */
  public static ParameterSet create(Object[][] values) {
    return create(values, false);
  }

  /**
   * Creates a parameter set based on the object array.  Rows are iterations, columns are parameters.  If labels are
   * included, the first column is treated as the name of the iteration and not a parameter.
   * @param values the parameter set values
   * @param labelsIncluded specifies if the array includes labels
   * @return the created parameter set
   * @see org.xj4.parameterized.set.MultiValueParameterSet
   */
  public static ParameterSet create(Object[][] values, boolean labelsIncluded) {
    String[] labels = null;
    if(labelsIncluded) {
      labels = new String[values.length];
      Object[][] newValues = new Object[values.length][];
      for(int i = 0; i < values.length; i++) {
        labels[i] = values[i].length > 0 ? values[i][0].toString() : Integer.toString(i);
        newValues[i] = ArrayUtils.subarray(values[i], 1, values[i].length);
      }
      values = newValues;
    }
    return validate(new MultiValueParameterSet(labels, values));
  }

  /**
   * Creates a parameter set for a method with one parameter based on the object array.  Each value is an iteration.
   * @param values the parameter values
   * @return the created parameter set
   * @see org.xj4.parameterized.set.SingleValueParameterSet
   */
  public static ParameterSet create(Object[] values) {
    return validate(new SingleValueParameterSet(values));
  }

  /**
   * Creates a parameter set for a method with one parameter based on the object array and the label array.  Each value
   * is an iteration.
   * @param values the parameter values
   * @param labels the iteration labels
   * @return the created parameter set
   * @see org.xj4.parameterized.set.SingleValueParameterSet
   */
  public static ParameterSet create(Object[] values, String[] labels) {
    return validate(new SingleValueParameterSet(values, labels));
  }

  /**
   * Creates a parameter set for a method with one parameter and one iteration.
   * @param value the parameter value
   * @return the created parameter set
   * @see org.xj4.parameterized.set.SingleValueParameterSet
   */
  public static ParameterSet create(Object value) {
    return create(new Object[] { value });
  }

  /**
   * Creates a parameter set for a method with one parameter and one iteration with the given label.
   * @param value the parameter value
   * @param label the iteration label
   * @return the created parameter set
   * @see org.xj4.parameterized.set.SingleValueParameterSet
   */
  public static ParameterSet create(Object value, String label) {
    return create(new Object[] { value }, new String[] { label });
  }

  /**
   * Appends multiple parameter sets together.  Each set should have the same number of parameters, this simply adds
   * iterations from each set and combines them into one.
   *
   * @param subSets the subsets
   * @return the created parameter set
   * @see org.xj4.parameterized.set.AppendParameterSet
   */
  public static ParameterSet append(ParameterSet... subSets) {
    return validate(new AppendParameterSet(subSets));
  }

  /**
   * Merges multiple parameter sets with the same number of iterations.  The first iteration will include the parameters
   * of the first iteration in each subset, and so on.
   * @param subSets the subsets
   * @return the created parameter set
   * @see org.xj4.parameterized.set.SimpleMergeParameterSet
   */
  public static ParameterSet simpleMerge(ParameterSet... subSets) {
    return validate(new SimpleMergeParameterSet(subSets));
  }

  /**
   * Merges multiple parameter sets, while explicitly defining which subset's labels should carry over.
   * @param labelsFrom the index of the subset to use labels from
   * @param subSets the subsets
   * @return the created parameter set
   * @see #simpleMerge(ParameterSet...)
   * @see org.xj4.parameterized.set.SimpleMergeParameterSet 
   */
  public static ParameterSet simpleMerge(int labelsFrom, ParameterSet... subSets) {
    ParameterSet labelSource = subSets[labelsFrom];
    String[] labelArray = new String[labelSource.getNumIterations()];
    for(int i = 0; i < labelArray.length; i++) {
      labelArray[i] = labelSource.getIterationLabel(i);
    }
    return validate(new SimpleMergeParameterSet(labelArray, subSets));
  }

  /**
   * Creates a new parameter set that has the same number of iterations as the subset, and one parameter - the index of
   * the iteration.
   * @param subSet the subset
   * @return the created parameter set
   * @see org.xj4.parameterized.set.IndexParameterSet
   */
  public static ParameterSet index(ParameterSet subSet) {
    return validate(new IndexParameterSet(subSet));
  }

  /**
   * Creates an index set and then merges it to the beginning of the subset.
   * @param subSet the subset
   * @return the created parameter set
   * @see org.xj4.parameterized.set.IndexParameterSet
   * @see org.xj4.parameterized.set.SimpleMergeParameterSet
   */
  public static ParameterSet indexAndMerge(ParameterSet subSet) {
    return simpleMerge(1, index(subSet), subSet);
  }

  /**
   * Combines multiple parameter sets such that each iteration of each subset is combined with each iteration of every
   * other subset.
   * @param subSets the subsets
   * @return the created parameter set
   * @see org.xj4.parameterized.set.CartesianParameterSet
   */
  public static ParameterSet cartesian(ParameterSet... subSets) {
    return validate(new CartesianParameterSet(subSets));
  }

  /**
   * Validates that the parameter set was properly created.  Should be called on every created parameter set.
   * @param definition the parameter set to validate
   * @return the parameter set
   */
  protected static ParameterSet validate(ParameterSet definition) {
    definition.validate();
    return definition;
  }
}
