/**
 * Copyright 2008 Peach Jean Solutions
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.xj4.parameterized;

import org.xj4.XJ4TestClass;
import org.xj4.spi.MethodHandler;
import static org.xj4.parameterized.InternalParameterSetUtils.*;
import org.apache.commons.lang.ArrayUtils;
import org.apache.commons.lang.ClassUtils;
import org.junit.runners.model.FrameworkMethod;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

/**
 * Method handler for parameterized methods.  Takes a single test method, combines it with a parameter set, and produces
 * a test method for each iteration of the parameter set.
 *
 * @author jbunting
 */
public class ParameterizedMethodHandler implements MethodHandler {

  private XJ4TestClass testClass;

  public ParameterizedMethodHandler(XJ4TestClass testClass) {
    this.testClass = testClass;
  }

  /**
   * Returns true if the test method is parameterized.
   * @param method the method to handle
   * @return true if parameterized, false otherwise
   */
  public boolean canHandle(FrameworkMethod method) {
    return isParameterized(method.getMethod());
  }

  /**
   * Generates a list of framework methods, one for each iteration of the parameter set, that will invoke the method
   * with the parameters given in that iteration.
   * @param method the method as created by JUnit
   * @return the list of methods
   */
  public List<FrameworkMethod> handle(FrameworkMethod method) {
    ParameterSet parameterSet = parameterSetOf(method.getMethod());
    List<FrameworkMethod> frameworkMethods = new ArrayList<FrameworkMethod>();
    for(int i = 0; i < parameterSet.getNumIterations(); i++) {
      frameworkMethods.add(new ParameterizedFrameworkMethod(method.getMethod(), parameterSet, i));
    }
    return frameworkMethods;
  }

  /**
   * Ensure that the method takes the same number of parameters as supplied by the parameter set, and that they are of
   * the same types.
   * 
   * @param method the method generated by JUnit
   * @param errors a list of validation errors - any new ones should be added to this list
   */
  public void validate(FrameworkMethod method, List<Throwable> errors) {
    method.validatePublicVoid(false, errors);

    try {
      // will throw InvalidParameterSetException if something is wrong with the parameter set
      ParameterSet parameterSet = parameterSetOf(method.getMethod());
      // checkin to ensure that the parameter set provides the same number of parameters as the method requires
      int numMethodParameters = method.getMethod().getParameterTypes().length;
      if(!(numMethodParameters == parameterSet.getIterationSize())) {
        errors.add(new Exception(String.format(
                "Parameterized method %s takes %d parameters - but parameter set %s provides %d parameters.",
                method.getName(), numMethodParameters, parameterSet.getName(), parameterSet.getIterationSize())));
        return; // there's no point in checking the types if the number of parameters is wrong
      }
      // check that all of the types in the parameter set are correct  
      Class<?>[] types = method.getMethod().getParameterTypes();
      for(int i = 0; i < parameterSet.getNumIterations(); i++) {
        Object[] params = parameterSet.getIteration(i);
        for(int j = 0; j < types.length; j++) {
          if(!isAssignable(getActualParameterType(params[j]), types[j])) {
            errors.add(new Exception(String.format(
                    "On test method %s, parameter %d of iteration %d is of type %s and the method requires %s.",
                    method.getName(), j, i, getActualParameterType(params[j]).getName(), types[j]
            )));
          }
        }
      }
    } catch (InvalidParameterSetException e) {
      errors.add(e);
    }
  }

  /**
   * Framework method that invokes a method with the parameters in a given iteration of a given parameter set.
   */
  private class ParameterizedFrameworkMethod extends FrameworkMethod {

    private ParameterSet parameterSet;
    private int iteration;

    public ParameterizedFrameworkMethod(Method method, ParameterSet parameterSet, int iteration) {
      super(method);
      this.parameterSet = parameterSet;
      this.iteration = iteration;
    }

    /**
     * Gets the name of the test method.  Provides for descriptive failure reporting.
     * @return the name
     */
    @Override
    public String getName() {
      return String.format("%s[%s]", super.getName(), parameterSet.getIterationLabel(iteration));
    }

    /**
     * Invokes the test method.  Provided args are prepended to the parameter set's list.
     * @param target the target object
     * @param args method arguments
     * @return the return value of the method
     * @throws Throwable if there is an error
     */
    @Override
    public Object invokeExplosively(Object target, Object ... args) throws Throwable {
      return super.invokeExplosively(target, buildArgumentList(target, parameterSet.getIteration(iteration), args));
    }

    private Object[] buildArgumentList(Object target, Object[] parameters, Object[] inputArgs) {
      Object[] args = ArrayUtils.addAll(inputArgs, parameters);
      for(int i = inputArgs.length; i < args.length; i++) {
        if(args[i] instanceof ParameterSource) {
          args[i] = ((ParameterSource)args[i]).generate(target);
        }
      }
      return args;
    }
  }

  private boolean isAssignable(Class type, Class toType) {
    return ClassUtils.isAssignable(forceFromPrimitive(type), forceFromPrimitive(toType));
  }

  private Class<?> forceFromPrimitive(Class<?> type) {
    if(type != null && type.isPrimitive()) {
      return ClassUtils.primitiveToWrapper(type);
    } else {
      return type;
    }
  }
}
