/**
 * Copyright 2008 Peach Jean Solutions
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.xj4.parameterized.set;

import org.apache.commons.lang.NullArgumentException;
import org.xj4.parameterized.ParameterSet;

/**
 * Combines multiple parameter sets such that each iteration of each subset is combined with each iteration of every
 * other subset.  If set 1 has x parameters and i iterations, and set 2 has y parameters and j iterations, the resulting
 * Cartesian set will have (x+y) parameters and i * j iterations.
 *
 * @author jbunting
 */
public class CartesianParameterSet extends AbstractCombinedParameterSet {

  public CartesianParameterSet(ParameterSet... subSets) {
    super(null, null, subSets);
  }

  public CartesianParameterSet(String name, String[] labelArray, ParameterSet... subSets) {
    super(name, labelArray, subSets);
  }

  public int getNumIterations() {
    int numIterations = 0;

    for(ParameterSet subSet : getSubDefinitions()) {
      if(subSet.getIterationSize() > 0) {
        if(numIterations == 0) {
          numIterations = 1;
        }
        numIterations *= subSet.getIterationSize();
      }
    }
    return numIterations;
  }

  public int getIterationSize() {
    int iterationSize = 0;
    for(ParameterSet subSet : getSubDefinitions()) {
      iterationSize += subSet.getIterationSize();
    }
    return iterationSize;
  }

  public void validate() {
    if(getSubDefinitions().length < 1) {
      throw new IllegalArgumentException("At least one sub definition must be specified!");
    }
    for(ParameterSet definition : getSubDefinitions()) {
      if(definition == null) {
        throw new NullArgumentException("Sub defintions cannot be null!");
      }
    }
  }

  protected String getLabelSeparator() {
    return "*";
  }

  protected int getSubDefinitionIteration(int iteration, int subDefinitionIndex) {
    for(int i = 0; i < subDefinitionIndex; i++) {
      iteration /= getSubDefinitions()[subDefinitionIndex].getNumIterations();
    }
    return iteration % getSubDefinitions()[subDefinitionIndex].getNumIterations();
  }
}
